<?php

/**
 * @file
 * Implements help hooks for the module.
 */

use Drupal\Core\Routing\RouteMatchInterface;

use Drupal\formatter_suite\Branding;

/**
 * Implements hook_help().
 *
 * This function responds to the help route for the Formatter Suite module
 * and provides a single page with multiple sections to document the
 * purpose and uses of the module.
 */
function formatter_suite_help(
  string $routeName,
  RouteMatchInterface $routeMatch) {
  //
  // The function returns a render array containing multiple help sections,
  // partly mandated by Drupal best practices.
  //
  // Check the route.
  if ($routeName !== 'help.page.formatter_suite') {
    // The route is not part of our help page. Do nothing.
    return;
  }

  // Get module information.
  $moduleTitle = \Drupal::moduleHandler()->getName('formatter_suite');
  $imagePath = '/' . drupal_get_path('module', 'formatter_suite') . '/images/';

  //
  // About
  // -----
  // This section is recommended by Drupal best practices.
  $help = '';
  $help .= '<h4>' . t('About') . '</h4>';
  $help .= '<p>' . t(
    '<strong>@moduleTitle</strong> provides a suite of field formatters to help present numbers, dates, times, text, images, and entity references. The suite contains the following formatters, organized by the field types they support:',
    [
      '@moduleTitle' => $moduleTitle,
    ]) . '</p>';

  $help .= <<<EOS
<ul class="formatter_suite-toc">
  <li>Date & time<ul>
    <li><a href="#DateTimeList">Date & time list</a></li>
    <li><a href="#DateTimeCustomList">Custom date & time list</a></li>
    <li><a href="#DateTimeTimeAgoList">Time ago list</a></li>
  </ul></li>
  <li>Email<ul>
    <li><a href="#GeneralEmail">General email address</a></li>
    <li><a href="#GeneralEmailList">General email address list</a></li>
  </ul></li>
  <li>Entity reference<ul>
    <li><a href="#GeneralEntityReference">General entity reference</a></li>
    <li><a href="#GeneralEntityReferenceList">General entity reference list</a></li>
    <li><a href="#EntityReferenceRenderList">Rendered entity list</a></li>
  </ul></li>
  <li>Image<ul>
    <li><a href="#ImageEmbedData">Image with embeded data URL</a></li>
  </ul></li>
  <li>Numbers (Decimal, Float, and Integer)<ul>
    <li><a href="#NumberWithBytes">Bytes with KB/MB/GB suffix</a></li>
    <li><a href="#GeneralNumber">General number</a></li>
    <li><a href="#GeneralNumberList">General number list</a></li>
    <li><a href="#GeneralNumberWithBarIndicator">General number with bar indicator</a></li>
    <li><a href="#GeneralNumberWithMinMax">General number with min/max</a></li>
  </ul></li>
  <li>Text (String long, Text, Text long, and Text with Summary)<ul>
    <li><a href="#TextWithExpandCollapseButtons">Text with expand/collapse buttons</a></li>
  </ul></li>
  <li>Timestamp<ul>
    <li><a href="#TimestampList">Timestamp list</a></li>
    <li><a href="#TimestampTimeAgoList">Time ago list</a></li>
  </ul></li>
</ul>
EOS;

  //
  // Uses
  // ----
  // This section is recommended by Drupal best practices.
  $help .= '<h4>' . t('Uses') . '</h4>';
  $help .= '<p>' . t(
    "Web site adminstrators may use the <strong>@moduleTitle</strong> module, along with the core <strong>Field UI</strong> and <strong>View UI</strong> modules, to configure the way in which field data is presented. The module's formatters offer a rich set of features that go beyond core formatters to provide detailed control over number notation styles, email address presentation, date and time lists, and more. Special formatters add graphic indicators, such as horizontal bar indicators showing numeric extent. Other formatters add interactivity to field values, such as expand/collapse buttons that show and hide large blocks of text.",
    [
      '@moduleTitle' => $moduleTitle,
    ]) . '</p>';

  //
  // Formatters
  // ----------
  // List the formatters.
  //
  // Datetime.
  $help .= '<h4>' . t('Formatters for dates and times') . '</h4>';
  $help .= '<dl class="formatter_suite-formatter-list">';

  // -------------------------------------------------------------------.
  $help .= '<a name="DateTimeList"></a>';
  $help .= '<dt>' . t('Date & time list') . '</dt>';
  $image = '<img src="' . $imagePath . 'DateTimeList.png">';
  $help .= '<dd><em>' . $image . t('Supports: Any Date & Time field.') . '</em></dd>';
  $help .= '<dd>' . t('A formatter based on the core <strong>Default</strong> formatter that also supports formatting multi-value fields as a list.') . '</dd>';
  $help .= '<dd>' . t('Settings include all settings from the <strong>Default</strong> formatter, plus those to select a list style and optional list separator, like a comma. List styles include numbered, bulleted, and unbulleted multi-line lists, and a single line list.') . '</dd>';

  // -------------------------------------------------------------------.
  $help .= '<a name="DateTimeCustomList"></a>';
  $help .= '<dt>' . t('Custom date & time list') . '</dt>';
  $image = '<img src="' . $imagePath . 'DateTimeCustomList.png">';
  $help .= '<dd><em>' . $image . t('Supports: Any Date & Time field.') . '</em></dd>';
  $help .= '<dd>' . t('A formatter based on the core <strong>Custom</strong> formatter that also supports formatting multi-value fields as a list.') . '</dd>';
  $help .= '<dd>' . t('Settings include all settings from the <strong>Custom</strong> formatter, plus those to select a list style and optional list separator, like a comma. List styles include numbered, bulleted, and unbulleted multi-line lists, and a single line list.') . '</dd>';

  // -------------------------------------------------------------------.
  $help .= '<a name="DateTimeTimeAgoCustomList"></a>';
  $help .= '<dt>' . t('Time ago list') . '</dt>';
  $image = '<img src="' . $imagePath . 'DateTimeTimeAgoList.png">';
  $help .= '<dd><em>' . $image . t('Supports: Any Date & Time field.') . '</em></dd>';
  $help .= '<dd>' . t('A formatter based on the core <strong>Time ago</strong> formatter that also supports formatting multi-value fields as a list.') . '</dd>';
  $help .= '<dd>' . t('Settings include all settings from the <strong>Time ago</strong> formatter, plus those to select a list style and optional list separator, like a comma. List styles include numbered, bulleted, and unbulleted multi-line lists, and a single line list.') . '</dd>';

  $help .= '</dl>';

  //
  // Email address.
  $help .= '<h4>' . t('Formatters for email addresses') . '</h4>';
  $help .= '<dl class="formatter_suite-formatter-list">';

  // -------------------------------------------------------------------.
  $help .= '<a name="GeneralEmail"></a>';
  $help .= '<dt>' . t('General email address') . '</dt>';
  $image = '<img src="' . $imagePath . 'GeneralEmail.png">';
  $help .= '<dd><em>' . $image . t('Supports: Any Email field.') . '</em></dd>';
  $help .= '<dd>' . t('A general-purpose formatter for email address fields.') . '</dd>';
  $help .= '<dd>' . t('Settings select whether to display the email address as plain text, as the address with a "mailto" link, or as a "mailto" link with custom text, such as "Email me" or "Contact us". "Mailto" links typically cause a web browser to start up the user\'s email application when clicked.') . '</dd>';

  // -------------------------------------------------------------------.
  $help .= '<a name="GeneralEmailList"></a>';
  $help .= '<dt>' . t('General email address list') . '</dt>';
  $image = '<img src="' . $imagePath . 'GeneralEmailList.png">';
  $help .= '<dd><em>' . $image . t('Supports: Any Email field.') . '</em></dd>';
  $help .= '<dd>' . t('A formatter based on the <strong>General email address</strong> formatter that also supports formatting multi-value fields as a list.') . '</dd>';
  $help .= '<dd>' . t('Settings include all settings from the <strong>General email address</strong> formatter, plus those to select a list style and optional list separator, like a comma. List styles include numbered, bulleted, and unbulleted multi-line lists, and a single line list.') . '</dd>';

  $help .= '</dl>';

  //
  // Entity references.
  $help .= '<h4>' . t('Formatters for entity references') . '</h4>';
  $help .= '<dl class="formatter_suite-formatter-list">';

  // -------------------------------------------------------------------.
  $help .= '<a name="GeneralEntityReference"></a>';
  $help .= '<dt>' . t('General entity reference') . '</dt>';
  $image = '<img src="' . $imagePath . 'GeneralEntityReference.png">';
  $help .= '<dd><em>' . $image . t('Supports: Any Entity Reference field, including for Files, Images, Taxonomy terms, and Users.') . '</em></dd>';
  $help .= '<dd>' . t('A general-purpose formatter for entity reference fields for entities of any type, including those for files, images, taxonomy terms, and user accounts.') . '</dd>';
  $help .= '<dd>' . t("Settings select whether to display the referenced entity's ID, title, or a title linked to the entity.") . '</dd>';

  // -------------------------------------------------------------------.
  $help .= '<a name="GeneralEntityReferenceList"></a>';
  $help .= '<dt>' . t('General entity reference list') . '</dt>';
  $image = '<img src="' . $imagePath . 'GeneralEntityReferenceList.png">';
  $help .= '<dd><em>' . $image . t('Supports: Any Entity Reference field, including for Files, Images, Users, and Taxonomy terms.') . '</em></dd>';
  $help .= '<dd>' . t('A formatter based on the <strong>General entity reference</strong> formatter that also supports formatting multi-value fields as a list.') . '</dd>';
  $help .= '<dd>' . t('Settings include all settings from the <strong>General entity reference</strong> formatter, plus those to select a list style and optional list separator, like a comma. List styles include numbered, bulleted, and unbulleted multi-line lists, and a single line list.') . '</dd>';

  // -------------------------------------------------------------------.
  $help .= '<a name="EntityReferenceRenderList"></a>';
  $help .= '<dt>' . t('Rendered entity list') . '</dt>';
  $image = '<img src="' . $imagePath . 'EntityReferenceRenderList.png">';
  $help .= '<dd><em>' . $image . t('Supports: Any Entity Reference field, including for Files, Images, Users, and Taxonomy terms.') . '</em></dd>';
  $help .= '<dd>' . t('A formatter based on the core <strong>Render</strong> entity reference formatter that also supports formatting multi-value fields as a list.') . '</dd>';
  $help .= '<dd>' . t('Settings include all settings from the <strong>Render</strong> formatter, plus those to select a list style and optional list separator, like a comma. List styles include numbered, bulleted, and unbulleted multi-line lists, and a single line list.') . '</dd>';

  $help .= '</dl>';

  // Images.
  $help .= '<h4>' . t('Formatters for images') . '</h4>';
  $help .= '<dl class="formatter_suite-formatter-list">';

  // -------------------------------------------------------------------.
  $help .= '<a name="ImageEmbedData"></a>';
  $help .= '<dt>' . t('Image with embedded data URL') . '</dt>';
  $image = '<img src="' . $imagePath . 'ImageEmbedData.png">';
  $help .= '<dd><em>' . $image . t('Supports: Any Image field.') . '</em></dd>';
  $help .= '<dd>' . t("A formatter based on the core <strong>Image</strong> formatter that is aimed at presenting small images by embeding them directly within the page using a data URL instead of a file URL. This can improve page loading times by reducing the number of files a browser has to request before it can present a page. This works best for small images, such as thumbnails.") . '</dd>';
  $help .= '<dd>' . t("Settings include all settings from the <strong>Image</strong> formatter, plus those to set the maximum image width and height for conversion into an data URL.") . '</dd>';

  $help .= '</dl>';

  // Numeric.
  $help .= '<h4>' . t('Formatters for numbers') . '</h4>';
  $help .= '<dl class="formatter_suite-formatter-list">';

  // -------------------------------------------------------------------.
  $help .= '<a name="NumberWithBytes"></a>';
  $help .= '<dt>' . t('Bytes with KB/MB/GB suffix') . '</dt>';
  $image = '<img src="' . $imagePath . 'NumberWithBytes.png">';
  $help .= '<dd><em>' . $image . t('Supports: Any Decimal, Float, or Integer field.') . '</em></dd>';
  $help .= '<dd>' . t('A formatter specifically for fields that store the size of a quantity of digital data measured in bytes. This value could be a file size or the amount of memory required.') . '</dd>';
  $help .= '<dd>' . t('Settings select whether values are summarized as a quantity using the international standard for <em>Kilobytes</em> (1000 bytes = 1 KB) or legacy <em>Kibibytes</em> (1024 bytes = 1 KiB). Values may be expressed with a specified number of decimal digits, and suffixes may use abbreviations, like "KB", or full words, like "Kilobyte".') . '</dd>';

  // -------------------------------------------------------------------.
  $help .= '<a name="GeneralNumber"></a>';
  $help .= '<dt>' . t('General number') . '</dt>';
  $image = '<img src="' . $imagePath . 'GeneralNumber.png">';
  $help .= '<dd><em>' . $image . t('Supports: Any Decimal, Float, or Integer field.') . '</em></dd>';
  $help .= '<dd>' . t('A general-purpose formatter for numeric values, including support for percentages and scientific notation. Formatting features are similar to those found in spreadsheet applications for formatting numeric cells.') . '</dd>';
  $help .= '<dd>' . t('Settings vary based upon a notation style:');
  $help .= '<ul>';
  $help .= '<li>' . t('<em>Basic number:</em> Formats numbers using basic settings controlling the number of decimal places and whether a thousands separator (,) is included.') . '</li>';
  $help .= '<li>' . t('<em>General number:</em> Formats numbers with detailed settings controlling the number of decimal places, the decimal separator, whether a thousands separator is included, the thousands separator, whether to pad with zeroes, the padding width, whether to show the plus sign on positive values, and whether to show the minus sign or parenthesis on negative values.') . '</li>';
  $help .= '<li>' . t('<em>Numeral system:</em> Formats numbers using a selected base, such as base 2 for binary, 8 for octal, or 16 for hex. Settings control the base and optional zero padding.') . '</li>';
  $help .= '<li>' . t('<em>Percentage:</em> Formats numbers as percentages by multiplying them by 100 and adding a percentage sign (%). Settings control the number of decimal places and whether a thousands separator (,) is included.') . '</li>';
  $help .= '<li>' . t('<em>Scientific:</em> Formats numbers using scientific notation and settings controlling the exponent style (E-notation or superscript), the number of decimal places, and whether a thousands separator (,) is included.') . '</li>';
  $help .= '</ul>';
  $help .= t("All general number notation styles support including the field's prefix and suffix before and after the value. The prefix is often used for currency symbols, while the suffix is often used for units of measure.");
  $help .= '</dd>';

  // -------------------------------------------------------------------.
  $help .= '<a name="GeneralNumberList"></a>';
  $help .= '<dt>' . t('General number list') . '</dt>';
  $image = '<img src="' . $imagePath . 'GeneralNumberList.png">';
  $help .= '<dd><em>' . $image . t('Supports: Any Decimal, Float, or Integer field.') . '</em></dd>';
  $help .= '<dd>' . t('A formatter based on the <strong>General number</strong> formatter that also supports formatting multi-value fields as a list.') . '</dd>';
  $help .= '<dd>' . t('Settings include all settings from the <strong>General number</strong> formatter, plus those to select a list style and optional list separator, like a comma. List styles include numbered, bulleted, and unbulleted multi-line lists, and a single line list.') . '</dd>';

  // -------------------------------------------------------------------.
  $help .= '<a name="GeneralNumberWithBarIndicator"></a>';
  $help .= '<dt>' . t('General number with bar indicator') . '</dt>';
  $image = '<img src="' . $imagePath . 'GeneralNumberWithBarIndicator.png">';
  $help .= '<dd><em>' . $image . t('Supports: Any Decimal, Float, or Integer field.') . '</em></dd>';
  $help .= '<dd>' . t("A formatter based on the <strong>General number</strong> formatter that adds a colored horizontal bar beside the number. The bar's length varies based upon the field's value and its distance between the field's minimum and maximum values.") . '</dd>';
  $help .= '<dd>' . t("Settings include all settings from the <strong>General number</strong> formatter, plus those to set a maximum bar length, a bar width, and the bar and background color. The field's value can be shown before or after the bar, or omitted.") . '</dd>';

  // -------------------------------------------------------------------.
  $help .= '<a name="GeneralNumberWithMinMax"></a>';
  $help .= '<dt>' . t('General number with min/max') . '</dt>';
  $image = '<img src="' . $imagePath . 'GeneralNumberWithMinMax.png">';
  $help .= '<dd><em>' . $image . t('Supports: Any Decimal, Float, or Integer field.') . '</em></dd>';
  $help .= '<dd>' . t("A formatter based on the <strong>General number</strong> formatter that also shows the field's minimum and maximum values, using a variety of styles.") . '</dd>';
  $help .= '<dd>' . t('Settings include all settings from the <strong>General number</strong> formatter, plus those to select among common formats for integrating the field\'s minimum and maximum with the value, such as "0 &le; 5 &le; 10", "5 out of 10", "5/10", and others. Formatted output also can be customized with a token-like syntax to place the minimum and maximum values along with the field\'s prefix and suffix.') . '</dd>';

  $help .= '</dl>';

  // Text.
  $help .= '<h4>' . t('Formatters for text') . '</h4>';
  $help .= '<dl class="formatter_suite-formatter-list">';

  // -------------------------------------------------------------------.
  $help .= '<a name="TextWithExpandCollapseButtons"></a>';
  $help .= '<dt>' . t('Text with expand/collapse buttons') . '</dt>';
  $image = '<img src="' . $imagePath . 'TextWithExpandCollapseButtons.png">';
  $help .= '<dd><em>' . $image . t('Supports: Any String long, Text, Text long, or Text with Summary field.') . '</em></dd>';
  $help .= '<dd>' . t('A general-purpose formatter that presents long text values within an area shortened in height to a selected number of pixels, points, or ems. Expand and collapse buttons expand the text to full length, and return it to its shortened form.') . '</dd>';
  $help .= '<dd>' . t('Settings select the shortened text height and the labels for the expand and collapse buttons. Expand and collapse both use a brief animation to slide the area open or closed. The length of that animation, in milliseconds, can be set.') . '</dd>';

  $help .= '</dl>';

  // Timestamp.
  $help .= '<h4>' . t('Formatters for timestamps') . '</h4>';
  $help .= '<dl class="formatter_suite-formatter-list">';

  // -------------------------------------------------------------------.
  $help .= '<a name="TimestampList"></a>';
  $help .= '<dt>' . t('Timestamp list') . '</dt>';
  $image = '<img src="' . $imagePath . 'TimestampList.png">';
  $help .= '<dd><em>' . $image . t('Supports: Any Timestamp field, including Created and Changed dates.') . '</em></dd>';
  $help .= '<dd>' . t('A formatter based on the core <strong>Default</strong> formatter that also supports formatting multi-value fields as a list.') . '</dd>';
  $help .= '<dd>' . t('Settings include all settings from the <strong>Default</strong> formatter, plus those to select a list style and optional list separator, like a comma. List styles include numbered, bulleted, and unbulleted multi-line lists, and a single line list.') . '</dd>';

  // -------------------------------------------------------------------.
  $help .= '<a name="TimestampTimeAgoList"></a>';
  $help .= '<dt>' . t('Time ago list') . '</dt>';
  $image = '<img src="' . $imagePath . 'TimestampTimeAgoList.png">';
  $help .= '<dd><em>' . $image . t('Supports: Any Timestamp field, including Created and Changed dates.') . '</em></dd>';
  $help .= '<dd>' . t('A formatter based on the core <strong>Time ago</strong> formatter that also supports formatting multi-value fields as a list.') . '</dd>';
  $help .= '<dd>' . t('Settings include all settings from the <strong>Time ago</strong> formatter, plus those to select a list style and optional list separator, like a comma. List styles include numbered, bulleted, and unbulleted multi-line lists, and a single line list.') . '</dd>';

  $help .= '</dl>';

  //
  // Render element
  // --------------
  // Return a render element instead of the HTML string itself. This is
  // needed in order to include module libraries to style the help page.
  $page = [
    '#attached'  => [
      'library'  => [
        'formatter_suite/formatter_suite.help',
      ],
    ],
    'page'       => [
      '#type'    => 'container',
      '#tree'    => TRUE,
      '#attributes' => [
        'class'     => ['formatter_suite-admin-help'],
      ],
      'branding' => Branding::getBannerBranding(),
      'help'     => [
        '#type'  => 'html_tag',
        '#tag'   => 'div',
        '#value' => $help,
      ],
    ],
  ];

  return $page;
}
