<?php

namespace Drupal\formatter_suite\Plugin\Field\FieldFormatter;

use Drupal\Core\Entity\EntityInterface;
use Drupal\Core\Entity\Exception\UndefinedLinkTemplateException;
use Drupal\Core\Field\FieldItemListInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Field\Plugin\Field\FieldFormatter\EntityReferenceFormatterBase;

use Drupal\formatter_suite\Branding;

/**
 * Formats an entity reference.
 *
 * The entity reference may be shown as:
 *   - The entity ID.
 *   - The entity title.
 *   - The entity title and a link to the entity.
 *
 * @internal
 * This class is loosely based upon the Drupal core classes for formatting
 * entity references:
 *   - EntityReferenceIdFormatter
 *   - EntityReferenceLabelFormatter
 * @endinternal
 *
 * @ingroup formatter_suite
 *
 * @FieldFormatter(
 *   id          = "formatter_suite_general_entity_reference",
 *   label       = @Translation("Formatter Suite - General entity reference"),
 *   weight      = 1000,
 *   field_types = {
 *     "entity_reference",
 *   }
 * )
 */
class GeneralEntityReferenceFormatter extends EntityReferenceFormatterBase {

  /*---------------------------------------------------------------------
   *
   * Configuration.
   *
   *---------------------------------------------------------------------*/

  /**
   * Returns an array of formatting styles.
   *
   * @return string[]
   *   Returns an associative array with internal names as keys and
   *   human-readable translated names as values.
   */
  protected static function getEntityReferenceStyles() {
    return [
      'id'        => t('Entity ID'),
      'title'     => t('Entity title'),
      'titlelink' => t('Entity title with link'),
    ];
  }

  /**
   * {@inheritdoc}
   */
  public static function defaultSettings() {
    return array_merge(
      [
        'entityReferenceStyle' => 'titlelink',
      ],
      parent::defaultSettings());
  }

  /**
   * {@inheritdoc}
   */
  public function settingsSummary() {
    $this->sanitizeSettings();
    $entityReferenceStyle = $this->getSetting('entityReferenceStyle');
    $entityReferenceStyles = $this->getEntityReferenceStyles();

    return [$entityReferenceStyles[$entityReferenceStyle]];
  }

  /*---------------------------------------------------------------------
   *
   * Settings form.
   *
   *---------------------------------------------------------------------*/

  /**
   * Returns a brief description of the formatter.
   *
   * @return string
   *   Returns a brief translated description of the formatter.
   */
  protected function getDescription() {
    return $this->t("Formats an entity reference by showing the referenced entity's unique ID, title, or the title linked to the entity.");
  }

  /**
   * {@inheritdoc}
   */
  public function settingsForm(array $form, FormStateInterface $formState) {
    // Get the parent's form.
    $elements = parent::settingsForm($form, $formState);

    // Add branding.
    $elements = Branding::addFieldFormatterBranding($elements);
    $elements['#attached']['library'][] =
      'formatter_suite/formatter_suite.fieldformatter';

    // Below, some checkboxes and select choices show/hide other form
    // elements. We use Drupal's obscure 'states' feature, which adds
    // Javascript to elements to auto show/hide based upon a set of
    // simple conditions.
    //
    // Those conditions need to reference the form elements to check
    // (e.g. a checkbox), but the element ID and name are automatically
    // generated by the parent form. We cannot set them, or predict them,
    // so we cannot use them. We could use a class, but this form may be
    // shown multiple times on the same page, so a simple class would not be
    // unique. Instead, we create classes for this form only by adding a
    // random number marker to the end of the class name.
    $marker = rand();

    // Add description.
    //
    // Use a large negative weight to insure it comes first.
    $elements['description'] = [
      '#type'          => 'html_tag',
      '#tag'           => 'div',
      '#value'         => $this->getDescription(),
      '#weight'        => -1000,
      '#attributes'    => [
        'class'        => [
          'formatter_suite-settings-description',
        ],
      ],
    ];

    $weight = 0;

    // Prompt for each setting.
    $elements['entityReferenceStyle'] = [
      '#title'         => $this->t('Reference style'),
      '#type'          => 'select',
      '#options'       => $this->getEntityReferenceStyles(),
      '#default_value' => $this->getSetting('entityReferenceStyle'),
      '#weight'        => $weight++,
      '#wrapper_attributes' => [
        'class'        => [
          'formatter_suite-general-entity-reference-style',
        ],
      ],
      '#attributes'    => [
        'class'        => [
          'entityReferenceStyle-' . $marker,
        ],
      ],
    ];

    return $elements;
  }

  /**
   * Sanitize settings to insure that they are safe and valid.
   *
   * @internal
   * Drupal's class hierarchy for plugins and their settings does not
   * include a 'validate' function, like that for other classes with forms.
   * Validation must therefore occur on use, rather than on form submission.
   * @endinternal
   */
  protected function sanitizeSettings() {
    // Get settings.
    $entityReferenceStyle = $this->getSetting('entityReferenceStyle');
    $entityReferenceStyles = $this->getEntityReferenceStyles();

    // Sanitize & validate.
    if (empty($entityReferenceStyle) === TRUE ||
        isset($entityReferenceStyles[$entityReferenceStyle]) === FALSE) {
      $defaults = $this->defaultSettings();
      $entityReferenceStyle = $defaults['entityReferenceStyle'];
      $this->setSetting('entityReferenceStyle', $entityReferenceStyle);
    }
  }

  /*---------------------------------------------------------------------
   *
   * View.
   *
   *---------------------------------------------------------------------*/

  /**
   * {@inheritdoc}
   */
  protected function checkAccess(EntityInterface $entity) {
    switch ($this->getSetting('entityReferenceStyle')) {
      default:
      case 'id':
        return TRUE;

      case 'title':
      case 'titlelink':
        // Make sure we have access to the title.
        return $entity->access('view label', NULL, TRUE);
    }
  }

  /**
   * {@inheritdoc}
   */
  public function viewElements(FieldItemListInterface $items, $langcode) {
    switch ($this->getSetting('entityReferenceStyle')) {
      default:
      case 'id':
        // Show the entity ID.
        return $this->formatId($items, $langcode);

      case 'title':
        // Show the entity title.
        return $this->formatTitle($items, $langcode);

      case 'titlelink':
        // Show the entity title and a link to the entity.
        return $this->formatTitleLink($items, $langcode);
    }
  }

  /**
   * Format entity references as ids.
   *
   * @param \Drupal\Core\Field\FieldItemListInterface $items
   *   A list of field items to format.
   * @param string $langcode
   *   The language code.
   *
   * @return array
   *   Returns a render array for the formatted items.
   */
  protected function formatId(FieldItemListInterface $items, $langcode) {
    $elements = [];
    foreach ($this->getEntitiesToView($items, $langcode) as $delta => $entity) {
      if (empty($entity->id()) === TRUE) {
        continue;
      }

      // Security: The entity ID is generated by Drupal core and may be
      // assumed to be safe and free of any HTML or HTML entities.
      //
      // Including the ID as plain text sends it to a TWIG template
      // which uses {{ }} to show the value. This automatically sanitizes
      // the value to insure it does not contain any HTML.
      $elements[$delta] = [
        '#plain_text' => $entity->id(),
        '#cache'      => [
          'tags'      => $entity->getCacheTags(),
        ],
      ];
    }

    return $elements;
  }

  /**
   * Format entity references as titles.
   *
   * @param \Drupal\Core\Field\FieldItemListInterface $items
   *   A list of field items to format.
   * @param string $langcode
   *   The language code.
   *
   * @return array
   *   Returns a render array for the formatted items.
   */
  protected function formatTitle(FieldItemListInterface $items, $langcode) {
    $elements = [];
    foreach ($this->getEntitiesToView($items, $langcode) as $delta => $entity) {
      if (empty($entity->id()) === TRUE) {
        continue;
      }

      // Security: The entity label has been entered by the user and must
      // be assumed to be unsafe. It should not contain any HTML or HTML
      // entities.
      //
      // Including the label as plain text sends it to a TWIG template
      // which uses {{ }} to show the value. This automatically sanitizes
      // the value to insure it does not contain any HTML.
      $elements[$delta] = [
        '#plain_text' => $entity->label(),
        '#cache'      => [
          'tags'      => $entity->getCacheTags(),
        ],
      ];
    }

    return $elements;
  }

  /**
   * Format entity references as linked titles.
   *
   * @param \Drupal\Core\Field\FieldItemListInterface $items
   *   A list of field items to format.
   * @param string $langcode
   *   The language code.
   *
   * @return array
   *   Returns a render array for the formatted items.
   */
  protected function formatTitleLink(FieldItemListInterface $items, $langcode) {
    $elements = [];
    foreach ($this->getEntitiesToView($items, $langcode) as $delta => $entity) {
      if (empty($entity->id()) === TRUE) {
        continue;
      }

      $uri = NULL;
      if ($entity->isNew() === FALSE) {
        // Entity exists and should have a URI.
        try {
          $uri = $entity->urlInfo();
        }
        catch (UndefinedLinkTemplateException $e) {
          // The entity type doesn't have a way of returning a URI.
        }
      }

      if ($uri !== NULL) {
        // Security: The entity label has been entered by the user and must
        // be assumed to be unsafe. It should not contain any HTML or HTML
        // entities.
        //
        // Including the label as a title for a 'link' render type sends it
        // to a TWIG template which uses {{ }} to show the value. This
        // automatically sanitizes the value to insure it does not contain
        // any HTML.
        $elements[$delta] = [
          '#type'    => 'link',
          '#title'   => $entity->label(),
          '#url'     => $uri,
          '#options' => $uri->getOptions(),
          '#cache'   => [
            'tags'   => $entity->getCacheTags(),
          ],
        ];

        if (empty($items[$delta]->_attributes) === FALSE) {
          // There are item attributes. Add them to the link's options.
          $elements[$delta]['#options'] += [
            'attributes' => $items[$delta]->_attributes,
          ];

          // And remove them from the item since the are now included
          // on the link.
          unset($items[$delta]->_attributes);
        }
      }
      else {
        // No URI, so just output as plain text.
        //
        // Security: The entity label has been entered by the user and must
        // be assumed to be unsafe. It should not contain any HTML or HTML
        // entities.
        //
        // Including the label as plain text sends it to a TWIG template
        // which uses {{ }} to show the value. This automatically sanitizes
        // the value to insure it does not contain any HTML.
        $elements[$delta] = [
          '#plain_text' => $entity->label(),
          '#cache'      => [
            'tags'      => $entity->getCacheTags(),
          ],
        ];
      }
    }

    return $elements;
  }

}
